@@ -55,6 +55,7 @@
 #include "gpm-hal-monitor.h"
 #include "gpm-brightness.h"
 #include "gpm-tray-icon.h"
+#include "gpm-stock-icons.h"
 #include "gpm-manager.h"
 
 static void     gpm_manager_class_init (GpmManagerClass *klass);
@@ -72,6 +73,22 @@ typedef enum {
 	GPM_WARNING_ACTION = 5
 } GpmWarning;
 
+#define GPM_BUTTON_POWER		"power"
+#define GPM_BUTTON_SLEEP		"sleep"
+#define GPM_BUTTON_SUSPEND		"suspend"
+#define GPM_BUTTON_HIBERNATE		"hibernate"
+#define GPM_BUTTON_LID			"lid"
+#define GPM_BUTTON_BRIGHT_UP		"brightness-up"
+#define GPM_BUTTON_BRIGHT_DOWN		"brightness-down"
+#define GPM_BUTTON_BRIGHT_UP_DEP	"brightness_up"		/* Remove when we depend on HAL 0.5.8 */
+#define GPM_BUTTON_BRIGHT_DOWN_DEP	"brightness_down"	/* as these are the old names */
+#define GPM_BUTTON_LOCK			"lock"
+/* Using www until we get a better one defined for us by the kernel */
+#define GPM_BUTTON_BATTERY		"www"
+
+#define GPM_NOTIFY_TIMEOUT_LONG		20	/* seconds */
+#define GPM_NOTIFY_TIMEOUT_SHORT	5	/* seconds */
+
 struct GpmManagerPrivate
 {
 	GConfClient	*gconf_client;
@@ -94,6 +111,8 @@ struct GpmManagerPrivate
 	gboolean	 use_time_to_notify;
 	gboolean	 lid_is_closed;
 
+	const char	*reason;
+
 };
 
 enum {
@@ -129,9 +148,16 @@ G_DEFINE_TYPE (GpmManager, gpm_manager, G_TYPE_OBJECT)
 #define		BATTERY_CRITICAL_REMAINING_TIME		(5 * 60)  /* 5 minutes  */
 #define		BATTERY_ACTION_REMAINING_TIME		(2 * 60)  /* 2 minutes  */
 
+#define		LAPTOP_PANEL_DIM_BRIGHTNESS		30 /* % */
+
 #undef DISABLE_ACTIONS_FOR_TESTING
 /*#define DISABLE_ACTIONS_FOR_TESTING 1*/
 
+/* prototypes */
+static gboolean gpm_manager_suspend (GpmManager *manager, GError **error);
+static gboolean gpm_manager_hibernate (GpmManager *manager, GError **error);
+static gboolean gpm_manager_shutdown (GpmManager *manager, GError **error);
+
 GQuark
 gpm_manager_error_quark (void)
 {
@@ -215,22 +241,65 @@ gpm_manager_can_shutdown (GpmManager *manager,
 	return TRUE;
 }
 
-/* Return 0..8 dependending on percent */
-static gint
+/* Return index value dependending on percent
+	00-10  = 000
+	10-30  = 020
+	30-50  = 040
+	50-70  = 060
+	70-90  = 080
+	90-100 = 100
+*/
+static char *
 get_icon_index_from_percent (gint percent)
 {
-	const gint NUM_INDEX = 8;
-	gint	   index;
-
-	index = ((percent + NUM_INDEX / 2) * NUM_INDEX ) / 100;
-	if (index < 0) {
-		return 0;
-	} else if (index > NUM_INDEX) {
-		return NUM_INDEX;
+	gpm_debug ("percent = %i", percent);
+	if (percent < 10) {
+		return "000";
+	} else if (percent < 30) {
+		return "020";
+	} else if (percent < 50) {
+		return "040";
+	} else if (percent < 70) {
+		return "060";
+	} else if (percent < 90) {
+		return "080";
 	}
-	return index;
+	return "100";
 }
 
+/* required, as UPS and primary icons have charged, charging and discharging icons.
+   must free retval */
+static char *
+get_stock_id_helper (GpmPowerBatteryStatus *device_status, const char *prefix)
+{
+	char *index;
+	char *filename = NULL;
+
+	if (gpm_power_battery_is_charged (device_status)) {
+
+		filename = g_strdup_printf ("%s-charged", prefix);
+
+	} else if (device_status->is_charging) {
+
+		index = get_icon_index_from_percent (device_status->percentage_charge);
+		filename = g_strdup_printf ("%s-charging-%s", prefix, index);
+
+	} else if (device_status->is_discharging) {
+
+		index = get_icon_index_from_percent (device_status->percentage_charge);
+		filename = g_strdup_printf ("%s-discharging-%s", prefix, index);
+
+	} else {
+
+		/* We have a broken battery, not sure what to display here */
+		gpm_debug ("BROKEN BATTERY... Not sure what to do");
+		filename = g_strdup_printf ("%s-charging-000", prefix);
+	}
+	gpm_debug ("filename = %s", filename);
+	return filename;
+}
+
+/* must free retval */
 static char *
 get_stock_id (GpmManager *manager,
 	      int         icon_policy)
@@ -241,7 +310,7 @@ get_stock_id (GpmManager *manager,
 	GpmPowerBatteryStatus status_keyboard;
 	gboolean on_ac;
 	gboolean present;
-	int index;
+	gboolean use_fallback;
 
 	gpm_debug ("Getting stock icon for tray");
 
@@ -273,24 +342,22 @@ get_stock_id (GpmManager *manager,
 	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
 
 	/* we try CRITICAL: PRIMARY, UPS, MOUSE, KEYBOARD */
+	gpm_debug ("Trying CRITICAL: primary, ups, mouse, keyboard");
 	if (status_primary.is_present &&
 	    status_primary.percentage_charge < BATTERY_LOW_PERCENTAGE) {
-		index = get_icon_index_from_percent (status_primary.percentage_charge);
-		if (on_ac) {
-			return g_strdup_printf ("gnome-power-ac-%d-of-8", index);
-		} else {
-			return g_strdup_printf ("gnome-power-bat-%d-of-8", index);
-		}
+		return get_stock_id_helper (&status_primary, ICON_PREFIX_PRIMARY);
+
 	} else if (status_ups.is_present &&
 		   status_ups.percentage_charge < BATTERY_LOW_PERCENTAGE) {
-		index = get_icon_index_from_percent (status_ups.percentage_charge);
-		return g_strdup_printf ("gnome-power-ups-%d-of-8", index);
+		return get_stock_id_helper (&status_ups, ICON_PREFIX_UPS);
+
 	} else if (status_mouse.is_present &&
 		   status_mouse.percentage_charge < BATTERY_LOW_PERCENTAGE) {
-		return g_strdup_printf ("gnome-power-mouse");
+		return g_strdup_printf (GPM_STOCK_MOUSE_LOW);
+
 	} else if (status_keyboard.is_present &&
 		   status_keyboard.percentage_charge < BATTERY_LOW_PERCENTAGE) {
-		return g_strdup_printf ("gnome-power-keyboard");
+		return g_strdup_printf (GPM_STOCK_KEYBOARD_LOW);
 	}
 
 	if (icon_policy == GPM_ICON_POLICY_CRITICAL) {
@@ -299,18 +366,14 @@ get_stock_id (GpmManager *manager,
 	}
 
 	/* we try (DIS)CHARGING: PRIMARY, UPS */
+	gpm_debug ("Trying CHARGING: primary, ups");
 	if (status_primary.is_present &&
-	    (status_primary.is_charging || status_primary.is_discharging)) {
-		index = get_icon_index_from_percent (status_primary.percentage_charge);
-		if (on_ac) {
-			return g_strdup_printf ("gnome-power-ac-%d-of-8", index);
-		} else {
-			return g_strdup_printf ("gnome-power-bat-%d-of-8", index);
-		}
+	    (status_primary.is_charging || status_primary.is_discharging) ) {
+		return get_stock_id_helper (&status_primary, ICON_PREFIX_PRIMARY);
+
 	} else if (status_ups.is_present &&
-		   (status_ups.is_charging || status_ups.is_charging)) {
-		index = get_icon_index_from_percent (status_ups.percentage_charge);
-		return g_strdup_printf ("gnome-power-ups-%d-of-8", index);
+		   (status_ups.is_charging || status_ups.is_discharging) ) {
+		return get_stock_id_helper (&status_ups, ICON_PREFIX_UPS);
 	}
 
 	/* Check if we should just show the icon all the time */
@@ -320,24 +383,27 @@ get_stock_id (GpmManager *manager,
 	}
 
 	/* we try PRESENT: PRIMARY, UPS */
+	gpm_debug ("Trying PRESENT: primary, ups");
 	if (status_primary.is_present) {
-		index = get_icon_index_from_percent (status_primary.percentage_charge);
-		if (on_ac) {
-			if (!status_primary.is_charging && !status_primary.is_discharging) {
-				/* if we are not charging, then don't display the 'flash' */
-				return g_strdup ("gnome-power-ac-charged");
-			}
-			return g_strdup_printf ("gnome-power-ac-%d-of-8", index);
-		} else {
-			return g_strdup_printf ("gnome-power-bat-%d-of-8", index);
-		}
+		return get_stock_id_helper (&status_primary, ICON_PREFIX_PRIMARY);
+
 	} else if (status_ups.is_present) {
-		index = get_icon_index_from_percent (status_ups.percentage_charge);
-		return g_strdup_printf ("gnome-power-ups-%d-of-8", index);
+		return get_stock_id_helper (&status_ups, ICON_PREFIX_UPS);
 	}
 
-	/* we fallback to the ac_adapter icon */
-	return g_strdup_printf ("gnome-dev-acadapter");
+	/* we have a falback here, so that desktops by default do not have
+	   the icon, but it can be "forced" */
+	use_fallback = gconf_client_get_bool (manager->priv->gconf_client,
+					      GPM_PREF_USE_ICON_DESKTOPS,
+					      NULL);
+	if (use_fallback) {
+		/* we fallback to the ac_adapter icon */
+		gpm_debug ("Using fallback");
+		return g_strdup_printf (GPM_STOCK_AC_ADAPTER);
+	}
+	/* no icon! */
+	gpm_debug ("No devices present, and no fallback, so no icon displayed.");
+	return NULL;
 }
 
 static void
@@ -602,6 +668,14 @@ manager_policy_do (GpmManager *manager,
 		gpm_debug ("*ACTION* Shutdown");
 		gpm_manager_shutdown (manager, NULL);
 
+	} else if (strcmp (action, ACTION_INTERACTIVE) == 0) {
+
+		gpm_debug ("*ACTION* Interactive");
+		/* Log out interactively */
+		gnome_client_request_save (gnome_master_client (),
+					   GNOME_SAVE_GLOBAL,
+					   TRUE, GNOME_INTERACT_ANY, FALSE, TRUE);
+
 	} else {
 		gpm_warning ("unknown action %s", action);
 	}
@@ -663,7 +737,23 @@ gpm_manager_get_dpms_mode (GpmManager  *manager,
 	return ret;
 }
 
-gboolean
+/* we set the reason to be WHY. e.g. "user pressed hibernate button" */
+static void
+gpm_manager_set_reason (GpmManager *manager,
+			const char *reason)
+{
+	manager->priv->reason = reason;
+}
+
+/* we set the reason to be WHAT. e.g. "Hibernate system" */
+static void
+gpm_manager_log_reason (GpmManager *manager,
+			const char *action)
+{
+	gpm_syslog ("%s because %s", action, manager->priv->reason);
+}
+
+static gboolean
 gpm_manager_shutdown (GpmManager *manager,
 		      GError    **error)
 {
@@ -684,15 +774,15 @@ gpm_manager_shutdown (GpmManager *manager,
 				   GNOME_SAVE_GLOBAL,
 				   FALSE, GNOME_INTERACT_NONE, FALSE,  TRUE);
 
-	/* FIXME: make this return success/fail */
 	/* FIXME: make this async? */
+	gpm_manager_log_reason (manager, "Shutting down computer");
 	gpm_hal_shutdown ();
 	ret = TRUE;
 
 	return ret;
 }
 
-gboolean
+static gboolean
 gpm_manager_hibernate (GpmManager *manager,
 		       GError    **error)
 {
@@ -720,19 +810,25 @@ gpm_manager_hibernate (GpmManager *manager,
 	gpm_networkmanager_sleep ();
 
 	/* FIXME: make this async? */
+	gpm_manager_log_reason (manager, "Hibernating computer");
 	ret = gpm_hal_hibernate ();
 	if (! ret) {
 		char *message;
-		message = g_strdup_printf (_("HAL failed to %s. "
-					     "Check the <a href=\"%s\">FAQ page</a> for common problems."),
-					     _("hibernate"), GPM_FAQ_URL);
-
-		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-				      5000,
-				      _("Hibernate Problem"),
-				      NULL,
-				      message);
-		g_free (message);
+		gboolean show_notify;
+		/* We only show the HAL failed notification if set in gconf */
+		show_notify = gconf_client_get_bool (manager->priv->gconf_client,
+						     GPM_PREF_NOTIFY_HAL_ERROR, NULL);
+		if (show_notify) {
+			message = g_strdup_printf (_("HAL failed to %s. "
+						     "Check the <a href=\"%s\">FAQ page</a> for common problems."),
+						     _("hibernate"), GPM_FAQ_URL);
+			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
+					      GPM_NOTIFY_TIMEOUT_LONG,
+					      _("Hibernate Problem"),
+					      NULL,
+					      message);
+			g_free (message);
+		}
 	}
 
 	if (do_lock) {
@@ -743,7 +839,7 @@ gpm_manager_hibernate (GpmManager *manager,
 	return ret;
 }
 
-gboolean
+static gboolean
 gpm_manager_suspend (GpmManager *manager,
 		     GError    **error)
 {
@@ -771,19 +867,25 @@ gpm_manager_suspend (GpmManager *manager,
 	gpm_networkmanager_sleep ();
 
 	/* FIXME: make this async? */
+	gpm_manager_log_reason (manager, "Suspending computer");
 	ret = gpm_hal_suspend (0);
 	if (! ret) {
 		char *message;
-		message = g_strdup_printf (_("Your computer failed to %s.\n"
-					     "Check the <a href=\"%s\">FAQ page</a> for common problems."),
-					     _("suspend"), GPM_FAQ_URL);
-
-		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-				      5000,
-				      _("Suspend Problem"),
-				      NULL,
-				      message);
-		g_free (message);
+		gboolean show_notify;
+		/* We only show the HAL failed notification if set in gconf */
+		show_notify = gconf_client_get_bool (manager->priv->gconf_client,
+						     GPM_PREF_NOTIFY_HAL_ERROR, NULL);
+		if (show_notify) {
+			message = g_strdup_printf (_("Your computer failed to %s.\n"
+						     "Check the <a href=\"%s\">FAQ page</a> for common problems."),
+						     _("suspend"), GPM_FAQ_URL);
+			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
+					      GPM_NOTIFY_TIMEOUT_LONG,
+					      _("Suspend Problem"),
+					      NULL,
+					      message);
+			g_free (message);
+		}
 	}
 
 	if (do_lock) {
@@ -794,12 +896,50 @@ gpm_manager_suspend (GpmManager *manager,
 	return ret;
 }
 
+gboolean
+gpm_manager_suspend_dbus_method (GpmManager *manager,
+				 GError    **error)
+{
+	/* FIXME: From where? */
+	gpm_manager_set_reason (manager, "the DBUS method Suspend() was invoked");
+	return gpm_manager_suspend (manager, error);
+}
+
+gboolean
+gpm_manager_hibernate_dbus_method (GpmManager *manager,
+				   GError    **error)
+{
+	/* FIXME: From where? */
+	gpm_manager_set_reason (manager, "the DBUS method Hibernate() was invoked");
+	return gpm_manager_hibernate (manager, error);
+}
+
+gboolean
+gpm_manager_shutdown_dbus_method (GpmManager *manager,
+				  GError    **error)
+{
+	/* FIXME: From where? */
+	gpm_manager_set_reason (manager, "the DBUS method Shutdown() was invoked");
+	return gpm_manager_shutdown (manager, error);
+}
+
 static void
 idle_changed_cb (GpmIdle    *idle,
 		 GpmIdleMode mode,
 		 GpmManager *manager)
 {
 	GError  *error;
+	gboolean do_laptop_dim;
+
+	/* Ignore timeout events when the lid is closed, as the DPMS is
+	   already off, and we don't want to perform policy actions or re-enable
+	   the screen when the user moves the mouse on systems that do not
+	   support hardware blanking.
+	   Details are here: https://launchpad.net/malone/bugs/22522 */
+	if (manager->priv->lid_is_closed) {
+		gpm_debug ("lid is closed, so we are ignoring idle state changes");
+		return;
+	}
 
 	switch (mode) {
 	case GPM_IDLE_MODE_NORMAL:
@@ -812,6 +952,15 @@ idle_changed_cb (GpmIdle    *idle,
 			gpm_debug ("Unable to set DPMS active: %s", error->message);
 		}
 
+		/* Should we dim the screen? */
+		do_laptop_dim = gconf_client_get_bool (manager->priv->gconf_client,
+						       GPM_PREF_IDLE_DIM_SCREEN, NULL);
+		if (do_laptop_dim) {
+			/* resume to the previous brightness */
+			gpm_brightness_level_resume (manager->priv->brightness);
+		}
+
+		/* sync timeouts */
 		sync_dpms_policy (manager);
 
 		break;
@@ -826,6 +975,15 @@ idle_changed_cb (GpmIdle    *idle,
 			gpm_debug ("Unable to set DPMS active: %s", error->message);
 		}
 
+		/* Should we resume the screen? */
+		do_laptop_dim = gconf_client_get_bool (manager->priv->gconf_client,
+						       GPM_PREF_IDLE_DIM_SCREEN, NULL);
+		if (do_laptop_dim) {
+			/* save this brightness and dim the screen, fixes #328564 */
+			gpm_brightness_level_save (manager->priv->brightness,
+						   LAPTOP_PANEL_DIM_BRIGHTNESS);
+		}
+
 		/* sync timeouts */
 		sync_dpms_policy (manager);
 
@@ -834,6 +992,7 @@ idle_changed_cb (GpmIdle    *idle,
 		gpm_debug ("Idle state changed: SYSTEM");
 
 		/* can only be hibernate or suspend */
+		gpm_manager_set_reason (manager, "the system state is idle");
 		manager_policy_do (manager, GPM_PREF_SLEEP_TYPE);
 
 		break;
@@ -870,25 +1029,48 @@ dpms_mode_changed_cb (GpmDpms    *dpms,
 		       gpm_dpms_mode_to_string (mode));
 }
 
+static void
+battery_button_pressed (GpmManager *manager)
+{
+	char *message;
+
+	gpm_power_get_status_summary (manager->priv->power, &message, NULL);
+
+	gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
+			      GPM_NOTIFY_TIMEOUT_LONG,
+			      _("Power Information"),
+			      NULL,
+			      message);
+	g_free (message);
+}
+
 static void
 power_button_pressed (GpmManager   *manager,
 		      gboolean	    state)
 {
-	gpm_debug ("power button changed: %d", state);
-
-	/* Log out interactively */
-	gnome_client_request_save (gnome_master_client (),
-				   GNOME_SAVE_GLOBAL,
-				   TRUE, GNOME_INTERACT_ANY, FALSE,  TRUE);
+	gpm_debug ("power button pressed");
+	gpm_manager_set_reason (manager, "the power button has been pressed");
+	manager_policy_do (manager, GPM_PREF_BUTTON_POWER);
 }
 
 static void
 suspend_button_pressed (GpmManager   *manager,
 			gboolean      state)
 {
+	gpm_debug ("suspend button pressed");
+	gpm_manager_set_reason (manager, "the suspend button has been pressed");
 	manager_policy_do (manager, GPM_PREF_BUTTON_SUSPEND);
 }
 
+static void
+hibernate_button_pressed (GpmManager   *manager,
+			gboolean      state)
+{
+	gpm_debug ("hibernate button pressed");
+	gpm_manager_set_reason (manager, "the hibernate button has been pressed");
+	manager_policy_do (manager, GPM_PREF_BUTTON_HIBERNATE);
+}
+
 static void
 lid_button_pressed (GpmManager	 *manager,
 		    gboolean	  state)
@@ -897,7 +1079,7 @@ lid_button_pressed (GpmManager	 *manager,
 
 	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
 
-	gpm_debug ("button changed: %d", state);
+	gpm_debug ("lid button changed: %d", state);
 
 	/* We keep track of the lid state so we can do the 
 	   lid close on battery action if the ac_adapter is removed when the laptop
@@ -905,14 +1087,23 @@ lid_button_pressed (GpmManager	 *manager,
 	manager->priv->lid_is_closed = state;
 
 	if (state) {
+		/* Disable the screensaver, as we don't want this starting
+		   when the lid is shut */
+		gpm_screensaver_inhibit_activation ("gnome-power-manager has "
+						    "detected that the lid is closed");
 		if (on_ac) {
 			gpm_debug ("Performing AC policy");
+			gpm_manager_set_reason (manager, "the lid has been closed on ac power");
 			manager_policy_do (manager, GPM_PREF_AC_BUTTON_LID);
 		} else {
 			gpm_debug ("Performing battery policy");
+			gpm_manager_set_reason (manager, "the lid has been closed on battery power");
 			manager_policy_do (manager, GPM_PREF_BATTERY_BUTTON_LID);
 		}
 	} else {
+		/* re-enable the screensaver */
+		gpm_screensaver_allow_activation ();
+
 		/* we turn the lid dpms back on unconditionally */
 		gpm_manager_unblank_screen (manager, NULL);
 	}
@@ -921,42 +1112,39 @@ lid_button_pressed (GpmManager	 *manager,
 static void
 power_button_pressed_cb (GpmPower   *power,
 			 const char *type,
-			 const char *details,
 			 gboolean    state,
 			 GpmManager *manager)
 {
-	gpm_debug ("Received a button press event type=%s details=%s state=%d",
-		 type, details, state);
+	gpm_debug ("Button press event type=%s state=%d", type, state);
 
-	if (strcmp (type, "power") == 0) {
+	if (strcmp (type, GPM_BUTTON_POWER) == 0) {
 		power_button_pressed (manager, state);
 
-	} else if (strcmp (type, "sleep") == 0) {
+	} else if (strcmp (type, GPM_BUTTON_SLEEP) == 0) {
 		suspend_button_pressed (manager, state);
 
-	} else if (strcmp (type, "lid") == 0) {
+	} else if (strcmp (type, GPM_BUTTON_SUSPEND) == 0) {
+		suspend_button_pressed (manager, state);
+
+	} else if (strcmp (type, GPM_BUTTON_HIBERNATE) == 0) {
+		hibernate_button_pressed (manager, state);
+
+	} else if (strcmp (type, GPM_BUTTON_LID) == 0) {
 		lid_button_pressed (manager, state);
 
-	} else if (strcmp (type, "virtual") == 0) {
-		if (details == NULL) {
-			return;
-		}
+	} else if ((strcmp (type, GPM_BUTTON_BRIGHT_UP) == 0) ||
+		   (strcmp (type, GPM_BUTTON_BRIGHT_UP_DEP) == 0)) {
+		gpm_brightness_level_up (manager->priv->brightness);
 
-		if (strcmp (details, "BrightnessUp") == 0) {
-			gpm_brightness_level_up (manager->priv->brightness);
+	} else if ((strcmp (type, GPM_BUTTON_BRIGHT_DOWN) == 0) ||
+		   (strcmp (type, GPM_BUTTON_BRIGHT_DOWN_DEP) == 0)) {
+		gpm_brightness_level_down (manager->priv->brightness);
 
-		} else if (strcmp (details, "BrightnessDown") == 0) {
-			gpm_brightness_level_down (manager->priv->brightness);
+	} else if (strcmp (type, GPM_BUTTON_LOCK) == 0) {
+		gpm_screensaver_lock ();
 
-		} else if (strcmp (details, "Suspend") == 0) {
-			gpm_manager_suspend (manager, NULL);
-
-		} else if (strcmp (details, "Hibernate") == 0) {
-			gpm_manager_hibernate (manager, NULL);
-
-		} else if (strcmp (details, "Lock") == 0) {
-			gpm_screensaver_lock ();
-		}
+	} else if (strcmp (type, GPM_BUTTON_BATTERY) == 0) {
+		battery_button_pressed (manager);
 	}
 }
 
@@ -967,6 +1155,12 @@ power_on_ac_changed_cb (GpmPower   *power,
 {
 	gpm_debug ("Setting on-ac: %d", on_ac);
 
+	/* If we are on AC power we should show warnings again */
+	if (on_ac) {
+		gpm_debug ("Resetting last_primary_warning to NONE");
+		manager->priv->last_primary_warning = GPM_WARNING_NONE;
+	}
+
 	tray_icon_update (manager);
 
 	maybe_notify_on_ac_changed (manager, on_ac);
@@ -979,6 +1173,7 @@ power_on_ac_changed_cb (GpmPower   *power,
 	   when the laptop is closed and on battery. Fixes #331655 */
 	if (! on_ac && manager->priv->lid_is_closed) {
 		gpm_debug ("Doing battery policy when lid closed and power removed");
+		gpm_manager_set_reason (manager, "the lid has been closed, and the ac adapter removed");
 		manager_policy_do (manager, GPM_PREF_BATTERY_BUTTON_LID);
 	}
 }
@@ -1028,7 +1223,8 @@ battery_low_get_title (GpmWarning warning_type)
 {
 	char *title = NULL;
 
-	if (warning_type == GPM_WARNING_CRITICAL) {
+	if (warning_type == GPM_WARNING_ACTION ||
+	    warning_type == GPM_WARNING_CRITICAL) {
 
 		title = _("Power Critically Low");
 
@@ -1049,6 +1245,13 @@ battery_low_get_title (GpmWarning warning_type)
 	return title;
 }
 
+static gboolean
+manager_critical_action_do (GpmManager *manager)
+{
+	manager_policy_do (manager, GPM_PREF_BATTERY_CRITICAL);
+	return FALSE;
+}
+
 /* performs critical action is required, and displays notifications */
 static void
 battery_status_changed_primary (GpmManager	      *manager,
@@ -1064,11 +1267,6 @@ battery_status_changed_primary (GpmManager	      *manager,
 
 	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
 
-	if (battery_status->percentage_charge != manager->priv->last_primary_percentage_change) {
-		/* should we fire an event or something? */
-		manager->priv->last_primary_percentage_change = battery_status->percentage_charge;
-	}
-
 	/* If we are charging we should show warnings again as soon as we discharge again */
 	if (battery_status->is_charging) {
 		gpm_debug ("Resetting last_primary_warning to NONE");
@@ -1085,12 +1283,16 @@ battery_status_changed_primary (GpmManager	      *manager,
 
 		if (show_notify) {
 			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-					      5000,
+					      GPM_NOTIFY_TIMEOUT_SHORT,
 					      _("Battery Charged"),
 					      NULL,
 					      _("Your battery is now fully charged"));
 		}
-		return;
+	}
+
+	if (battery_status->percentage_charge != manager->priv->last_primary_percentage_change) {
+		/* should we fire an event or something? */
+		manager->priv->last_primary_percentage_change = battery_status->percentage_charge;
 	}
 
 	if (! battery_status->is_discharging) {
@@ -1112,21 +1314,48 @@ battery_status_changed_primary (GpmManager	      *manager,
 	}
 
 	if (warning_type == GPM_WARNING_ACTION) {
-		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-				      5000,
-				      _("Critical action"),
-				      NULL,
-				      _("The battery is below the critical level and "
-					"this computer is about to shutdown."));
+		const char *warning = NULL;
+		const char *action;
+
+		/* we have to do different warnings depending on the policy */
+		action = gconf_client_get_string (manager->priv->gconf_client,
+						  GPM_PREF_BATTERY_CRITICAL, NULL);
+
+		/* FIXME: we should probably convert to an ENUM type, and use that */
+		if (strcmp (action, ACTION_NOTHING) == 0) {
+			warning = _("The battery is below the critical level and "
+				    "this computer will <b>power-off</b> when the "
+				    "battery becomes completely empty.");
+		} else if (strcmp (action, ACTION_SUSPEND) == 0) {
+			warning = _("The battery is below the critical level and "
+				    "this computer is about to suspend.<br>"
+				    "<b>NOTE:</b> A small amount of power is required "
+				    "to keep your computer in a suspended state.");
+		} else if (strcmp (action, ACTION_HIBERNATE) == 0) {
+			warning = _("The battery is below the critical level and "
+				    "this computer is about to hibernate.");
+		} else if (strcmp (action, ACTION_SHUTDOWN) == 0) {
+			warning = _("The battery is below the critical level and "
+				    "this computer is about to shutdown.");
+		}
+
+		if (warning) {
+			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
+					      GPM_NOTIFY_TIMEOUT_LONG,
+					      _("Critical action"),
+					      NULL,
+					      warning);
+		}
+		gpm_manager_set_reason (manager, "we are critically low for the primary battery");
 		/* wait 10 seconds for user-panic */
-		g_usleep (1000 * 1000 * 10);
-		manager_policy_do (manager, GPM_PREF_BATTERY_CRITICAL);
+		g_timeout_add (1000*10, (GSourceFunc) manager_critical_action_do, manager);
 	}
 
 	/* Always check if we already notified the user */
 	if (warning_type > manager->priv->last_primary_warning) {
+		int timeout;
+
 		manager->priv->last_primary_warning = warning_type;
-		title = battery_low_get_title (warning_type);
 		remaining = gpm_get_timestring (battery_status->remaining_time);
 
 		/* Do different warnings for each GPM_WARNING */
@@ -1137,18 +1366,25 @@ battery_status_changed_primary (GpmManager	      *manager,
 			if (show_notify) {
 				message = g_strdup_printf (_("The AC Power has been unplugged. "
 						             "The system is now using battery power."));
+				timeout = GPM_NOTIFY_TIMEOUT_SHORT;
 			}
 		} else {
 			message = g_strdup_printf (_("You have approximately <b>%s</b> "
 						     "of remaining battery life (%d%%). "
 						     "Plug in your AC Adapter to avoid losing data."),
 						   remaining, battery_status->percentage_charge);
+				timeout = GPM_NOTIFY_TIMEOUT_LONG;
+		}
+		if (message) {
+			title = battery_low_get_title (warning_type);
+			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
+					      timeout,
+					      title,
+					      NULL,
+					      message);
+			g_free (message);
 		}
-		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-				      5000, title, NULL, message);
-
 		g_free (remaining);
-		g_free (message);
 	} else {
 		gpm_debug ("Already notified %i", warning_type);
 	}
@@ -1207,7 +1443,10 @@ battery_status_changed_ups (GpmManager	          *manager,
 						   remaining, battery_status->percentage_charge);
 		}
 		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-				      5000, title, NULL, message);
+				      GPM_NOTIFY_TIMEOUT_LONG,
+				      title,
+				      NULL,
+				      message);
 
 		g_free (remaining);
 		g_free (message);
@@ -1274,7 +1513,10 @@ battery_status_changed_misc (GpmManager	    	   *manager,
 				   name, battery_status->percentage_charge);
 
 	gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-			      5000, title, NULL, message);
+			      GPM_NOTIFY_TIMEOUT_LONG,
+			      title,
+			      NULL,
+			      message);
 
 	g_free (message);
 }
@@ -1382,6 +1624,7 @@ callback_gconf_key_changed (GConfClient *client,
 	gint	    brightness;
 	GpmManager *manager = GPM_MANAGER (user_data);
 	gboolean    on_ac;
+	gboolean enabled;
 
 	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
 
@@ -1435,6 +1678,13 @@ callback_gconf_key_changed (GConfClient *client,
 			gpm_brightness_level_set (manager->priv->brightness, brightness);
 		}
 
+	} else if (strcmp (entry->key, GPM_PREF_CAN_SUSPEND) == 0) {
+		gpm_manager_can_suspend (manager, &enabled, NULL);
+		gpm_tray_icon_enable_suspend (GPM_TRAY_ICON (manager->priv->tray_icon), enabled);
+
+	} else if (strcmp (entry->key, GPM_PREF_CAN_HIBERNATE) == 0) {
+		gpm_manager_can_hibernate (manager, &enabled, NULL);
+		gpm_tray_icon_enable_hibernate (GPM_TRAY_ICON (manager->priv->tray_icon), enabled);
 	}
 }
 
@@ -1444,6 +1694,7 @@ gpm_manager_tray_icon_hibernate (GpmManager   *manager,
 				 GpmTrayIcon  *tray)
 {
 	gpm_debug ("Received hibernate signal from tray icon");
+	gpm_manager_set_reason (manager, "user clicked hibernate from tray menu");
 	gpm_manager_hibernate (manager, NULL);
 }
 
@@ -1452,6 +1703,7 @@ gpm_manager_tray_icon_suspend (GpmManager   *manager,
 			       GpmTrayIcon  *tray)
 {
 	gpm_debug ("Received supend signal from tray icon");
+	gpm_manager_set_reason (manager, "user clicked suspend from tray menu");
 	gpm_manager_suspend (manager, NULL);
 }
 #endif
@@ -1470,6 +1722,8 @@ static void
 gpm_manager_init (GpmManager *manager)
 {
 	gboolean on_ac;
+	gboolean use_time;
+	gboolean check_type_cpu;
 #if ACTIONS_MENU_ENABLED
 	gboolean enabled;
 #endif
@@ -1510,6 +1764,12 @@ gpm_manager_init (GpmManager *manager)
 	g_signal_connect (manager->priv->idle, "changed",
 			  G_CALLBACK (idle_changed_cb), manager);
 
+	/* set up the check_type_cpu, so we can disable the CPU load check */
+	check_type_cpu = gconf_client_get_bool (manager->priv->gconf_client,
+						GPM_PREF_IDLE_CHECK_CPU,
+						NULL);
+	gpm_idle_set_check_cpu (manager->priv->idle, check_type_cpu);
+
 	manager->priv->dpms = gpm_dpms_new ();
 
 	gpm_debug ("creating new tray icon");
@@ -1541,19 +1801,33 @@ gpm_manager_init (GpmManager *manager)
 	g_signal_connect (manager->priv->dpms, "mode-changed",
 			  G_CALLBACK (dpms_mode_changed_cb), manager);
 
-	/* we don't want to be notified on coldplug if we are on battery power */
-	manager->priv->last_primary_warning = GPM_WARNING_DISCHARGING;
-	
 	/* we want all notifications */
 	manager->priv->last_ups_warning = GPM_WARNING_NONE;
 	manager->priv->last_mouse_warning = GPM_WARNING_NONE;
 	manager->priv->last_keyboard_warning = GPM_WARNING_NONE;
 	manager->priv->last_pda_warning = GPM_WARNING_NONE;
+	manager->priv->last_primary_warning = GPM_WARNING_NONE;
 	manager->priv->last_primary_percentage_change = 0;
 
-	/* We can change this easily if	this doesn't work in real-world
-	 * conditions, or perhaps make this a gconf configurable. */
-	manager->priv->use_time_to_notify = TRUE;
+	/* We don't want to be notified on coldplug if we are on battery power
+	   this should fix #332322 */
+	if (! on_ac) {
+		manager->priv->last_primary_warning = GPM_WARNING_DISCHARGING;
+	}
+
+	/* we set the reason to be unknown */
+	gpm_manager_set_reason (manager, "of an unknown reason");
+
+	/* We can disable this if the ACPI BIOS is fucked, and the
+	   time_remaining is therefore inaccurate or just plain wrong. */
+	use_time = gconf_client_get_bool (manager->priv->gconf_client,
+					  GPM_PREF_USE_TIME_POLICY, NULL);
+	if (use_time) {
+		gpm_debug ("Using per-time notification policy");
+	} else {
+		gpm_debug ("Using percentage notification policy");
+	}
+	manager->priv->use_time_to_notify = use_time;
 }
 
 static void
